@@ -127,15 +127,31 @@ class MonthRetention:
         return result
 
 class Pubnum:
-    def __init__(self, data, start_month='start_month', lag_month='lag_month', cp_num='cp_num', pub_num='pub_num'):
+    def __init__(self, data, start_month='start_month', lag_month='lag_month', cp_num='cp_num', pub_num='pub_num',
+                 import_cp='cp_num1',pub_back_num='pub_back_num'):
+        '''
+        :param data: dataFrame,包含字段引入月、观察月、引入人数、活跃人数、非回溯发文数、回溯发文数
+        :param start_month: 引入月列名
+        :param lag_month: 观察月列名
+        :param cp_num: 活跃人数（非回溯）列名
+        :param pub_num: 非回溯发文数列名
+        :param import_cp: 引入人数列名
+        :param pub_back_num: 回溯发文数列名
+        '''
         self.data = data
         self.data['avg_pub'] = self.data[pub_num] / self.data[cp_num]
         self.start_month = start_month
         self.lag_month = lag_month
         self.pub_num = pub_num
         self.cp_num = cp_num
+        self.import_cp = import_cp
+        self.pub_back_num = pub_back_num
+        self.data['avg_back_pub'] = self.data[pub_back_num] / self.data[import_cp]
 
     def avgpub_his(self):
+        '''
+        :return:返回各月引入作者非回溯发文均值，当月引入作者在单月非回溯发文取历史值，当月引入作者在其他月份的非回溯发文取最近的6个月数据
+        '''
         df = pd.pivot(self.data, index=self.lag_month, columns=self.start_month, values='avg_pub')
         result = pd.DataFrame(index=df.columns, columns=['m0_pub_num', 'avg_pub'])
         for col in df.columns:
@@ -144,6 +160,35 @@ class Pubnum:
         result = result.fillna(method='ffill')
         return result, df
 
+    def avgpub_back_his(self):
+        '''
+        :return:返回当月引入作者当月回溯发文量和之前引入作者在当月的回溯发文量
+        '''
+        df = pd.pivot(self.data, index=self.start_month,columns=self.lag_month, values='avg_back_pub')
+        result = pd.DataFrame(index=df.columns,columns=['new_cp_back_pub','old_cp_back_pub'])
+        for col in df.columns:
+            n = int(str(col)[4:]) - 2
+            last_month = '%04d%02d' % (int(int(str(col)[0:4]) + n // 12), int(n % 12 + 1))
+            last_month = int(last_month)
+            result.loc[col,'new_cp_back_pub'] = df.loc[col,col]
+            result.loc[col,'old_cp_back_pub'] = self.data[(self.data[self.lag_month]==col) &
+                                                          (self.data[self.start_month] != col)][self.pub_back_num].sum() / \
+                                                self.data[(self.data[self.lag_month]==col) &
+                                                          (self.data[self.start_month]==last_month)][self.import_cp].max()
+        return result
+
+    def avg_backpub_predict(self, pre_month):
+        '''
+        预测各月回溯发文人均发文量，用最近的6个月取平均得到预测值
+        :param pre_month: 预测月份
+        :return:返回预测的月回溯量，包括新作者回溯和老作者回溯量
+        '''
+        r = self.avgpub_back_his()
+        if pre_month in r.index:
+            return r.loc[pre_month]
+        else:
+            return r.iloc[-6:].mean(axis=0)
+
     def month_range(self, start_month, end_month):
         '''
         计算两个月份间的所有月份列表
@@ -178,7 +223,8 @@ class Pubnum:
 
 class MonthPubCalc:
     def __init__(self, data, total_cp:pd.Series,
-                 start_month='start_month', lag_month='lag_month', cp_num='cp_num', pub_num='pub_num', pr='pr'):
+                 start_month='start_month', lag_month='lag_month', cp_num='cp_num', pub_num='pub_num', pr='pr',
+                 import_cp='cp_num1',pub_back_num='pub_back_num'):
         self.data = data
         self.start_month = start_month
         self.lag_month = lag_month
@@ -186,7 +232,8 @@ class MonthPubCalc:
         self.cp_num = cp_num
         self.pr = pr
         self.total_cp = total_cp.copy()
-        self.data['total_cp'] = self.data['cp_num'] / self.data['pr']
+        self.import_cp = import_cp
+        self.pub_back_num = pub_back_num
 
     def calc_month_pubnum(self,pre_month):
         '''
@@ -195,11 +242,18 @@ class MonthPubCalc:
         :return: 预测月份的发文数
         '''
         retention = MonthRetention(self.data, start_month=self.start_month, lag_month=self.lag_month,pr=self.pr,cp_num=self.cp_num)
-        avgpub = Pubnum(self.data,start_month=self.start_month,lag_month=self.lag_month,cp_num=self.cp_num,pub_num=self.pub_num)
+        avgpub = Pubnum(self.data,start_month=self.start_month,lag_month=self.lag_month,cp_num=self.cp_num,pub_num=self.pub_num
+                        ,import_cp=self.import_cp,pub_back_num=self.pub_back_num)
         retention_rate = retention.retention_predict_per_month(pre_month)
         avgpub_num = avgpub.avgpub_predict(pre_month)
+        avgback_num = avgpub.avg_backpub_predict(pre_month)
         pubnum = retention_rate[pre_month] * avgpub_num[pre_month] * self.total_cp
-        return pubnum.sum()
+        n = int(str(pre_month)[4:])-2
+        last_month = '%04d%02d' % (int(int(str(pre_month)[0:4]) + n // 12), int(n % 12 + 1))
+        last_month = int(last_month)
+        backnum = self.total_cp.loc[pre_month] * avgback_num.loc['new_cp_back_pub'] + \
+                    self.total_cp.loc[last_month] * avgback_num.loc['old_cp_back_pub']
+        return pubnum.sum(), backnum
 
     def month_range(self, start_month, end_month):
         '''
@@ -212,13 +266,18 @@ class MonthPubCalc:
         month_range = ['%04d%02d' % (int(int(str(start_month)[0:4]) + n // 12), int(n % 12 + 1))
                        for n in range(int(str(start_month)[4:])-1, int(str(start_month)[4:]) + months)]
         return month_range
+
     def calc_monthlist_pubnum(self,start,end):
         month_list = self.month_range(start, end)
         month_list = [int(month) for month in month_list]
-        result = pd.DataFrame(index=month_list, columns=['pubnum'], data=month_list)
-        result['pubnum'] = result.pubnum.apply(lambda x: self.calc_month_pubnum(x))
+        result = pd.DataFrame(index=month_list, columns=['pubnum', 'backnum'])
+        result['pubnum'] = month_list
+        result['backnum'] = month_list
+        result['pubnum'] = result.pubnum.apply(lambda x: self.calc_month_pubnum(x)[0])
+        result['backnum'] = result.backnum.apply(lambda x: self.calc_month_pubnum(x)[1])
         return result
 
+
 class IncomePredict:
     def __init__(self, df_his: pd.DataFrame, df_pre:pd.DataFrame, month:str, delta_days:str, vv_income:str, n=15, m=90):
         """
@@ -278,7 +337,7 @@ class IncomePredict:
         return result
 
 if __name__ == '__main__':
-    df = pd.read_csv('D:/项目/X项目/X发文预估模型/留存模型/X引入留存(月).csv', encoding='utf-8')
+    df = pd.read_csv('x引入留存11月.csv', encoding='utf-8')
     df['cp'] = df['cp_num'] / df['pr']
     total_cp = df.groupby('start_month')['cp'].max()
     df = df[(df.start_month <= 202209) & (df.lag_month <= 202209)]
